@@ -37,6 +37,116 @@ Vue.component( 'upload-field', {
     }
 } );
 
+Vue.component( 'foreign-key-field', {
+    template: '#foreign-key-field',
+    props: {
+        name: {
+            required: true
+        },
+        value: {
+            required: true
+        },
+        schemaName: {
+            required: true
+        },
+        displayField: { },
+        valueField: { },
+        disabled: {
+            type: 'boolean',
+            default: false
+        },
+    },
+    data: function () {
+        // XXX Replace with VueX
+        var schema = Yancy.schema[ this.schemaName ].operations.set.schema;
+        return {
+            _value: this.$props.value,
+            displayValue: 'Loading...',
+            loading: true,
+            uid: 'fk-' + this.$props.name + '-' + Math.floor( Math.random() * 10000 ),
+            searchQuery: '',
+            searchResults: [
+            ],
+            displayField: this.$props.displayField || (
+                schema['x-list-columns'] ? schema['x-list-columns'][0] : ( schema['x-id-field'] || 'id' )
+            ),
+            valueField: this.$props.valueField || schema['x-id-field'] || 'id'
+        };
+    },
+    mounted: function () {
+        var dropdownToggleEl = $( '#' + this.uid + ' .dropdown-toggle' );
+        dropdownToggleEl.dropdown({
+            boundary: 'viewport',
+        });
+        dropdownToggleEl.parent().on( 'shown.bs.dropdown', function (ev) {
+            var input = $( ev.target ).find( 'input[type=text]' );
+            // Without this setTimeout, the screen jumps significantly
+            // when focusing the first time (when the dropdown is
+            // initially created)
+            setTimeout( function () { input.focus() }, 10 );
+        } );
+        this.fetchDisplayValue();
+    },
+    beforeDestroy: function () {
+        var dropdownToggleEl = $( '#' + this.uid + ' .dropdown-toggle' );
+        dropdownToggleEl.parent().off( 'shown.bs.dropdown' );
+    },
+    methods: {
+        fetchDisplayValue: function () {
+            if ( !this.$data._value ) {
+                this.displayValue = '(none)';
+                return;
+            }
+            this.loading = true;
+            $.ajax({
+                method: 'GET',
+                url: specUrl + '/' + this.$props.schemaName + '/' + this.$data._value,
+                context: this
+            })
+            .then( this.updateDisplayValue );
+        },
+        updateDisplayValue: function ( item ) {
+            this.displayValue = item[ this.$data.displayField ];
+            this.loading = false;
+        },
+        showDropdown: function () {
+            var dropdownEl = $( '#' + this.uid );
+            dropdownEl.find( '.dropdown-toggle' ).dropdown('toggle');
+        },
+        submitSearch: function () {
+            $.ajax({
+                method: 'GET',
+                url: specUrl + '/' + this.$props.schemaName +
+                    '?' + this.$data.displayField + '=' + encodeURIComponent( this.searchQuery ),
+                context: this
+            })
+            .then( this.updateSearchResults );
+        },
+        updateSearchResults: function ( data ) {
+            var results = [];
+            for ( var i = 0; i < data.items.length; i++ ) {
+                results.push({
+                    display: data.items[i][ this.$data.displayField ],
+                    value: data.items[i][ this.$data.valueField ]
+                });
+            }
+            this.searchResults = results;
+            this.$nextTick( function () {
+                $( '#' + this.uid + ' .dropdown-toggle' ).dropdown('update');
+            } );
+        },
+        select: function (item) {
+            this.$data._value = item.value;
+            this.$emit( 'input', item.value );
+            this.updateDisplayValue( item );
+            this.searchResults = [];
+            this.searchQuery = '';
+            $( '#' + this.uid ).find( '.dropdown-toggle' ).dropdown('hide');
+            this.fetchDisplayValue();
+        }
+    }
+} );
+
 Vue.component('edit-field', {
     template: '#edit-field',
     props: {
@@ -71,7 +181,10 @@ Vue.component('edit-field', {
             schemaType = this.schema.type;
         }
 
-        if ( this.schema['enum'] ) {
+        if ( this.schema['x-foreign-key'] ) {
+            fieldType = 'foreign-key';
+        }
+        else if ( this.schema['enum'] ) {
             fieldType = 'select';
         }
         else if ( schemaType == 'boolean' ) {
